% Example 2.1 Cylindrical Reservoir with No Refilling Emptying through an Orifice
% S = Reservoir surface area of a cylindrical tank [m^2];
% H = Reservoir initial depth [m];
% A = Orifice surface area [m^2];
% Co = Discharge coefficient [dimensionless];
% Dt = Computational time step [s];
% Dz = Computational depth step [m];
% Qout(z) = Outflow discharge [m^3/s];
% z(t) = Numerical solution for the water depth [m];
% ns = number of horizontal sections;
clc; clear all; close all;
% Input data;
g=9.81;
ns=10;
H=10;
Dz=H/ns;
A=0.005;
Co=0.7;
Dt=5;
S=[5.0, 5.0, 5.0, 5.0, 5.0, 5.0, 5.0, 5.0, 5.0, 5.0, 5.0];
% Initial conditions;
z=H;
k=0;
% Main program;
for k=1:10000;
    if z<0
        z=0;
    end
    Qout=Co*A*sqrt(2*g*z);
    j=1;
    if z>j*Dz; j=j+1;
    end
    dH=j*Dz-z;
    S1=S(j+1)+(S(j)-S(j+1))*dH/Dt;
    znew=z-Qout*Dt/S1;
    z=znew;
    Z(k)=znew;
    % To facilitate plotting take 100*Qout;
    Qp(k)=Qout*100; m=k;
end
plot(1:m,Z,'b','Linewidth',1.5)
hold on
plot(1:m,Qp,'m','Linewidth',1.5)
v=[0, 450, 0, 10];
axis(v)
xlabel('Number of time steps'), ylabel('')
legend('z: Water stage [m]','Qout: Outflow discharge x 10E-2 [m^3/s]')
