% Example 2.2 Flood Routing Through a Reservoir
% Td = Flood duration [s];
% Qo = Initial flood discharge [m^3/s];
% zo = Initial water elevation in reservoir [m];
% zs = Spillway crest elevation [m];
% B = Spillway width [m];
% Dt = Time step [s];
% nm = Number of integration steps;
% ns = Values of reservoir surfaces at different elevations;
% nf = Values of flood hydrograph at different times;
clc;clear all;close all;
% Input data;
Td = 40000;
Qo = 2000;
zo = 50;
zs = 95;
B = 100;
Dt = 100;
nm = 1000;
z = zo;
t = 0;
% Initial conditions;
Qin = Qo;
Qout = 0;
% Main program;
for n = 1:nm;
    t = n*Dt;
    % Calculation of the inflow hydrograph;
    if Qin > 0
        Qin = Qo*(1-t/Td);
    else Qin = 0;
    end;
    % Calculation of the outflow discharge;
    if z > zs
        Qout = B*(z-zs)^1.5;
    else Qout = 0;
    end;
    % Reservoir horizontal plane area;
    SS = 100*z^2;
    % Calculation of the reservoir water elevation;
    z = z+Dt*(Qin-Qout)/SS;
    k=n;
    Qinp(k)=Qin;
    Qoutp(k)=Qout;
    zp(k)=z;
end;
plot(1:k,zp,'b','Linewidth',1.5);
hold on
plot(1:k,Qinp/10,'g','Linewidth',1.5)
plot(1:k,Qoutp/10,'m','Linewidth',1.5)
xlabel('Number of time steps')
text(100,180,'Inflow hydrograph x 10E-1 [m^3/s]')
text(370,52,'Outflow hydrograph x 10E-1 [m^3/s]')
text(500,105,'Water elevation [m]')

