% Example 2.3 Pollution in a Semi-enclosed Tidal Lagoon
% Qin = Pollutant discharge rate [m^3/s];
% Cin = Pollutant concentration rate at the source [g/m^3];
% Vol = Water volume of the lagoon [m^3];
% Qo = Maximum renewal discharge [m^3/s];
% Qout = Renewal discharge varying with the tide [m^3/s];
% Co = Initial pollutant concentration throughout the lagoon [g/m^3];
% Rb = Biodegradation rate [1/s];
% Qp = Pump discharge rate [m^3/s];
% Tp = Tidal period [s];
% Dt = Time step [s];
clc; clear all; close all;
% Input data
Qin = 0.1;
Cin = 500;
Vol = 50000;
Qo = 2;
Co = 1;
Rb = 0.000001;
Qp0 = 0;
Qp1 = 1;
Qp5 = 15;
Tp = 43200;
Dt = 300;
C0 = Co;
C1 = Co;
C5 = Co;
% Main program;
for n = 1:2000;
    t= n*Dt;
    % Different outflow pumping rates;
    Qout0 = Qo*abs(cos((2*pi/Tp)*t))+Qp0;
    Qout1 = Qo*abs(cos((2*pi/Tp)*t))+Qp1;
    Qout5 = Qo*abs(cos((2*pi/Tp)*t))+Qp5;
    % Calculation of the contaminant concentrations;
    Cnew0 = C0 + (Cin*Dt*Qin/Vol) - (C0*Dt*Qout0/Vol) - (Rb*Dt*C0);
    Cnew1 = C1 + (Cin*Dt*Qin/Vol) - (C1*Dt*Qout1/Vol) - (Rb*Dt*C1);
    Cnew5 = C5 + (Cin*Dt*Qin/Vol) - (C5*Dt*Qout5/Vol) - (Rb*Dt*C5);
    % Renewal of the contaminant concentrations;
    C0=Cnew0;
    C1=Cnew1;
    C5=Cnew5;
    m=n;
    Cplot0(m)=C0;
    Cplot1(m)=C1;
    Cplot5(m)=C5;
end
m=n;
Cplot0(m)=C0;
Cplot1(m)=C1;
Cplot5(m)=C5;
plot(1:m,Cplot0,'b')
hold on
plot(1:m,Cplot1,'g')
plot(1:m,Cplot5,'m')
xlabel('Number of time steps')
ylabel('Contaminant concentration [g/m^3]')
text(800,6, 'pumping rate 5 m^3/s')
text(800,18, 'pumping rate 1 m^3/s')
text(800,34, 'no pumping')