% Example 3.1 Two-Dimensional Potential Flow in a Rectangular Conduit
% f(i,j) = Velocity potential;
% SS = Potential source;
% im = Number of vertical nodes;
% jm = Number of horizontal nodes;
% iter = Number of iterations;
% fer = Difference in potential values between two subsequent iterations;
clc; clear all; close all;
% Input data;
im=30;
jm=60;
SS=10;
iter=1000;
% Initialization of the grid with zero potential values;
f(1:im,1:jm)=0;
is=im/2;
js=jm/2;
% Dirichlet type boundary conditions at the entrance and exit of the conduit;
f(1:im,1)=10;
f(is-1:is+1,jm)=0;
k1=0;
k2=0;
% Spatial allocation of the potential source;
S(im,jm)=0;
S(15,30)=SS;
% Main program;
k=0;
for kk=1:iter
    k=1+kk;
    fold=f;
    % Estimation of the potential values;
    for k1=2:im-1
        for k2=2:jm-1
            f(k1,k2)=(f(k1+1,k2)+f(k1-1,k2)+f(k1,k2-1)+f(k1,k2+1)+S(k1,k2))/4;
        end
    end
    % Von Neumann boundary conditions at the upper and lower conduit walls;
    for j1=1:jm
        f(1,j1)=f(2,j1);
        f(im,j1)=f(im-1,j1);
    end
    % Von Neumann boundary conditions at the end wall of the conduit;
    for j2=1:(is-3)
        f(j2,jm)=f(j2,jm-1);
    end
    for j2=is+3 : im
        f(j2,jm)=f(j2,jm-1);
    end
    % Estimation of the error between two consecutive iterations;
    fnew=f;
    fer=fnew-fold;
end
i=1:k1+1;
j=1:k2+1;
fpp=fnew(i,j);
error=fer(i,j);
% Plot of the potential distribution;
surf(j,i,fpp);
figure
% Plot of the computational error distribution;
surf(j,i,error);
