% Example 3.2 Two-Dimensional Couette Flow Caused by an Oscillating Plate
% U = Velocity amplitude [cm/s];
% T = Oscillation period [s];
% v = Kinematic viscosity of the fluid [cm^2/s];
% Dt = Time step [s];
% Dz = Vertical step [cm];
% nm = Number of vertical discretization layers;
% tm = Number of time steps;
clc; clear all; close all;
% Input data;
U = 1;
T = 500;
T1 = 1000;
v = 0.01;
v1 = 0.1;
Dt = 1;
Dz = 1;
nm = 19;
im = nm +1;
tm = 5000;
% Initialization of the velocity field;
for m=1:im
    u1(m)=0; un1=u1(m);
    u2(m)=0; un2=u2(m);
    u3(m)=0; un3=u3(m);
end
% Boundary condition at the fixed plate;
u1(im)=0;
u2(im)=0;
u3(im)=0;
% Main program;
for k=1:tm
    Dtt=Dt*k;
    % Boundary condition at the oscillating plate;
    u1(1)= U*sin(2*pi*Dtt/T);
    u2(1)= U*sin(2*pi*Dtt/T1);
    u3(1)= U*sin(2*pi*Dtt/T);
    % Calculation of the horizontal velocities between the two plates;
    for k1=2:nm
        un1(k1)=u1(k1)+(v*Dt/(Dz^2))*(u1(k1+1)-2*u1(k1)+u1(k1-1));
        un2(k1)=u2(k1)+(v*Dt/(Dz^2))*(u2(k1+1)-2*u2(k1)+u2(k1-1));
        un3(k1)=u3(k1)+(v1*Dt/(Dz^2))*(u3(k1+1)-2*u3(k1)+u3(k1-1));
        % Renewal of the velocity values;
        u1(k1)=un1(k1);
        u2(k1)=un2(k1);
        u3(k1)=un3(k1);
    end
    % Calculation of the shear stresses;
    ss1=((u1(1)-u1(2))/Dz)*v;
    tau1(k)=ss1;
    ss2=((u2(1)-u2(2))/Dz)*v;
    tau2(k)=ss2;
    ss3=((u3(1)-u3(2))/Dz)*v1;
    tau3(k)=ss3;
end
plot(1:tm,tau1,'r','Linewidth',1.5)
hold on
plot(1:tm,tau2,'g','Linewidth',1.5)
plot(1:tm,tau3,'b','Linewidth',1.5)
xlabel('Time steps'), ylabel('Shear stress along the oscillating plate [(cm/s)^2]')
axis([0, 5000, -0.049, 0.035])
legend('tau1:T=500s,v=0.01cm^2/s','tau2:T=1000s,v=0.01cm^2/s','tau3:T=500s,v=0.1cm^2/s',3)
