% Example 4.2 Water Hammer Analysis
% r = Pipe radius [m];
% c = Celeriy of the elastic wave [m/s];
% ho = upstream pressure head [m];
% fc = Pipe friction coefficient;
% tv = Valve closing time [s];
% Dt = Time step [s];
% Dx = Spatial step [m];
% nx = Number of discrete spatial points;
% nt = Number of time steps;
% u = Water velocity;
% h = Piezometric head;
clc; clear all; close all;
% Input data;
r=1;
c=500;
ho=5;
fc=0.05;
tv=5; tv1=20; tv2=40;
Dt=0.02;
Dx=50;
nx=100;
tn=10000;
g=9.81;
le=Dx*(nx-1);
% Initial values of u and h;
uo= sqrt(g*r*ho/2/le/fc);
for i=1:nx
    u(i)=uo;
    h(i)=ho-ho*(i-1)/(nx-1);
    hmax(i)=0;
    u1(i)=uo;
    h1(i)=ho-ho*(i-1)/(nx-1);
    h1max(i)=0;
    u2(i)=uo;
    h2(i)=ho-ho*(i-1)/(nx-1);
    h2max(i)=0;
end
for n=1:tn
    t=Dt*n;
    % Downstream condition for three vane closure times;
    if t<tv
        u(nx)=uo*(tv-t)/tv;
    end
    if t<tv1
        u1(nx)=uo*(tv1-t)/tv1;
    end
    if t<tv2
        u2(nx)=uo*(tv2-t)/tv2;
    end
    % Calculation of new water velocities;
    for k=2:nx-1
        ff=2*fc*u(k)*abs(u(k))/r;
        u(k)=u(k)-Dt/Dx*g*(h(k)-h(k-1))-Dt*ff;
        ff1=2*fc*u1(k)*abs(u1(k))/r;
        u1(k)=u1(k)-Dt/Dx*g*(h1(k)-h1(k-1))-Dt*ff1;
        ff2=2*fc*u2(k)*abs(u2(k))/r;
        u2(k)=u2(k)-Dt/Dx*g*(h2(k)-h2(k-1))-Dt*ff2;
    end
    % Calculation of piezometric head;
    for k=2:nx-1
        h(k)=h(k)-Dt/Dx*c^2/g*(u(k+1)-u(k));
        h1(k)=h1(k)-Dt/Dx*c^2/g*(u1(k+1)-u1(k));
        h2(k)=h2(k)-Dt/Dx*c^2/g*(u2(k+1)-u2(k));
        % Max piezometric head value for each pipe section;
        if h(k)>hmax(k)
            hmax(k)=h(k);
        end
        if h1(k)>h1max(k)
            h1max(k)=h1(k);
        end
        if h2(k)>h2max(k)
            h2max(k)=h2(k);
        end
        hend(n)=h(nx-1);
        h1end(n)=h1(nx-1);
        h2end(n)=h2(nx-1);
    end
end
hmax(nx)=hmax(nx-1);
h1max(nx)=h1max(nx-1);
h2max(nx)=h2max(nx-1);
plot(1:nx, hmax,'r','Linewidth',1.5)
hold on
plot(1:nx,h1max,'b','Linewidth',1.5)
plot(1:nx, h2max,'g','Linewidth',1.5)
xlabel('Number of spatial steps (Dx = 50m)'), ylabel('Max pressure head [m]')
legend('valve closure time = 5s','valve closing time = 20s', 'valve closing time = 40s')
axis([0,100,0,26])
figure,plot(1:tn,hend,'r','Linewidth',1.5)
hold on
plot(1:tn,h1end,'b','Linewidth',1.5)
plot(1:tn,h2end,'g','Linewidth',1.5)
xlabel('Time [s]')
ylabel('Pressure head at the end of the pipe [m]')
legend('valve closure time = 5s','valve closing time = 20s', 'valve closing time = 40s')