% Example 5.1 Water Surface Profiles
% Q = Volumetric discharge [m^3/s];
% S = Bed slope;
% Sf = Energy gradient;
% n = Manning's coefficient of friction;
% u = Flow velocity [m/s];
% b = Bottom width [m];
% m = Side slope;
% yn = Normal depth [m];
% yc = Critical depth [m};
% A = Cross sectioanl area {m^2];
% Rh = Hydraulic radius [m];
% Pw = Wetted perimeter [m];
% T = Surface width [m];
% L = Channel lenght [m];
% Y = Water depth [m];
% z= Bed elevation from the reference datum [m];
% h = Water elevation from the reference datum [m];
% Dx = Spatial step [m]:;
clc; clear all; close all;
% Input data;
g=9.81;
Q=25;
S=0.001;
n=0.025;
% Initial depth value to start the iterations;
yin=0.1;
% Trapezoidal, rectangular, triangular cross sections;
% Calculation of the normal depth;
for j=1:3
    b=[5 5 0];
    m=[1 0 1];
    y(1)=yin;
    i=1;
    dyn(1)=0.1;
    while (abs(dyn(i))>0.0001)
        An(i)=b(j)*y(i)+m(j)*(y(i))^2;
        Pwn(i)=b(j)+2*sqrt(m(j)^2+1)*y(i);
        Rhn(i)=An(i)/(Pwn(i));
        Tc(i)=b(j)+2*m(j)*y(i);
        fn(i)=sqrt(S)/n*(An(i)*Rhn(i)^(2/3))-Q;
        dfn(i)=sqrt(S)/n*((-4/3)*Rhn(i)^(5/3)*sqrt(1+m(j)^2)+5/3*Rhn(i)^(2/3)*Tc(i));
        y(i+1)=y(i)-fn(i)/dfn(i);
        dyn(i+1)=-fn(i)/dfn(i);
        i=i+1;
    end
    yn(j)=y(i);
    iter_yn(j)=i;
end
% Calculation of the critical depth;
for j=1:3
    yc(1)=yin;
    k=1;
    dyc(1)=0.1;
    while (abs(dyc(k))>0.0001)
        Ac(k)=b(j)*yc(k)+m(j)*(yc(k))^2;
        Tc(k)=b(j)+2*m(j)*yc(k);
        fc(k)=Ac(k)^(3/2)*Tc(k)^(-1/2)-Q/sqrt(g);
        dfc(k)=-m(j)*(Ac(k)/Tc(k))^(3/2)+(3/2)*sqrt(Ac(k)*Tc(k));
        %   dfc(k)=-m(j)*Ac(k)^(3/2)*Tc(k)^(-3/2)+(3/2)*Tc(k)^(-1/2)*Ac(k)^(1/2)*Tc(k);
        yc(k+1)=yc(k)-fc(k)/dfc(k);
        dyc(k+1)=-fc(k)/dfc(k);
        k=k+1;
    end
    ycr(j)=yc(k);
    iter_yc(j)=k;
end
% Water surface profile calculations for the trapezoidal channel;
Dx=1;
L=4000;
nx=fix(abs(L/Dx));
% Subcritical flow boundary condition;
% Water depth at the control section yc<yn<y: M1 profile;
Y =1.5*yn(1);
% Water depth at the control section yc<y<yn: M2 profile;
% Y =(yn(1)+ycr(1))/2;
% Supercritical flow boundary condition;
% Water depth at the control section yn<y<yc: S2 profile:
% Y=(yn(1)+ycr(1))/2;
% Water depth at the control section y<yn<yc: S3 profile;
% Y=0.1*yn(1);
if yn-ycr>0
    Dx=-Dx;
end
z(1)=0;
h(1)=Y(1);
b=b(1);
m=m(1);
% Main program;
for i=1:nx;
    A(i)=b*Y(i)+m*Y(i)^2;
    T(i)=b+2*m*Y(i);
    Pw(i)=b+2*Y(i)*sqrt(1+m^2);
    Rh(i)=A(i)/Pw(i);
    u(i)=Q/A(i);
    Sf(i)=n^2*u(i)^2/(Rh(i)^(4/3));
    % ODE f=dy/dx=(S-Sf)/(1-Fr^2);
    f(i)=(S-Sf(i))/(1-Q^2*T(i)/g/A(i)^3);
    yy(i)=Y(i)+0.5*f(i)*Dx;
    A(i)=b*yy(i)+m*yy(i)^2;
    T(i)=b+2*m*yy(i);
    Pw(i)=b+2*yy(i)*sqrt(1+m^2);
    Rh(i)=A(i)/Pw(i);
    u(i)=Q/A(i);
    Sf(i)=n^2*u(i)^2/(Rh(i)^(4/3));
    fnew(i)=(S-Sf(i))/(1-Q^2*T(i)/g/A(i)^3);
    Y(i+1)=Y(i)+fnew(i)*Dx;
    z(i+1)=-Dx*(i)*S;
    h(i+1)=z(i+1)+Y(i+1);
    x(i+1)=Dx*i;
end
plotyc=z+ycr(1);
plotyn=z+yn(1);
plot(x,z,'k','LineWidth',2); hold on;
plot (x,h,'b','LineWidth',2);
plot(x,plotyc,'r')
plot(x,plotyn,'g' )
legend('Bed','Water Surface','Critical Depth','Normal Depth',1);
xlabel('Distance [m]');
ylabel('Elevation [m]');
title('Water Surface Profile');


