% Example 5.2 Unsteady Flow in Rectangular Open Channel of Constant Width
% h = Flow depth [m];
% b = Channel width [m];
% S = Bed slope;
% C = Chezy coefficient of friction [m^(1/2)/s];
% Qin = Inflow hydrograph [m^3/s];
% Dx = Spatial step [m];
% Dt = Time step [s];
% nx = Number of spatial steps;
% nt = Number of time steps;
% dt = Time step of the inflow hydrograph [s];
% nth = Number of time steps of the inflow hydrograph;
clc; clear all; close all;
% Input data;
g=9.81;
Dx=1000;
Dt=5;
nx=41;
nt=9000;
dt=2000;
nth=15;
for i=1:nx
    h(i)=0;
    S(i)=0.001;
    b(i)=100;
    C(i)=50;
end
Qin=[0,600,400,200,100,50,0,0,0,0,0,0,0,0,0];
k=0;
% Main program;
for k=1:nt
    k=k+1;
    % Calculation of inflow discharge by interpolation;
    jj=k*Dt/dt;
    jr=round(jj)+1;
    if jr >= nth
        jr=nth-1;
    end
    Q(1)=Qin(jr)+(Qin(jr+1)-Qin(jr))*(k*Dt-(jr-1)*dt)/dt;
    % Calculation of the velocity and discharge;
    for i=2:nx-1
        if h(i-1)<0.001
            u(i)=0;
        else u(i)=u(i)+Dt*g*(S(i)-(h(i)-h(i-1))/Dx-u(i)*abs(u(i))/C(i)^2*2/(h(i)+h(i-1)));
        end
        Q(i)=u(i)*b(i)*(h(i)+h(i-1))/2;
    end
    % Downstream boundary condition;
    Q(nx)=b(nx)*h(nx-1)*sqrt(g*h(nx-1));
    % Estimation of the water depths:
    for i=1:nx-1
        h(i)=h(i)-Dt/Dx/(b(i)+b(i+1))*2*(Q(i+1)-Q(i));
    end
    % Data stored at different time intervals for plotting;
    if k == round(nt/9)
        for m=1:nx
            h1t(m)=h(m);
            Q1t(m)=Q(m);
        end
    end
    if k == round(nt/6)
        for m=1:nx
            h2t(m)=h(m);
            Q2t(m)=Q(m);
        end
    end
    if k == round(nt/4.5)
        for m=1:nx
            h3t(m)=h(m);
            Q3t(m)=Q(m);
        end
    end
    if k == round(nt/3)
        for m=1:nx
            h4t(m)=h(m);
            Q4t(m)=Q(m);
        end
    end
    if k == round(nt/2)
        for m=1:nx
            h5t(m)=h(m);
            Q5t(m)=Q(m);
        end
    end
    if k == nt
        for m=1:nx
            h6t(m)=h(m);
            Q6t(m)=Q(m);
        end
    end
end
plot(1:nth,Qin,'k','Linewidth',1.5)
xlabel('Number of time steps x 2000 [s]')
ylabel('Inflow hydrograph [m^3/s]')
axis([0,15,0,660])
figure, plot(1:nx,Q1t,'b','Linewidth',1.5)
hold on
plot(1:nx,Q2t,'r','Linewidth',1.5)
plot(1:nx,Q3t,'g','Linewidth',1.5)
plot(1:nx,Q4t,'b--','Linewidth',1.5)
plot(1:nx,Q5t,'r--','Linewidth',1.5)
plot(1:nx,Q6t,'g--','Linewidth',1.5)
xlabel('Longitudinal distance x 1000 [m]')
ylabel('Water discharge [m^3/s]')
axis([0,42,0,550])
legend('t=1000*Dt','t=1500*Dt','t=2000*Dt','t=3000*Dt','t=4500*Dt','t=9000*Dt')
figure, plot(1:nx,h1t,'b','Linewidth',1.5)
hold on
plot(1:nx,h2t,'r','Linewidth',1.5)
plot(1:nx,h3t,'g','Linewidth',1.5)
plot(1:nx,h4t,'b--','Linewidth',1.5)
plot(1:nx,h5t,'r--','Linewidth',1.5)
plot(1:nx,h6t,'g--','Linewidth',1.5)
xlabel('Longitudinal distance x 1000 [m]')
ylabel('Water depth [m]')
axis([0,42,0,2.6])
legend('t=1000*Dt','t=1500*Dt','t=2000*Dt','t=3000*Dt','t=4500*Dt','t=9000*Dt')
