% Example 5.3 Unsteady Flow in Rectangular Open Channel of Variable Width
% h = Flow depth [m];
% B = Channel width [m];
% S = Bed slope;
% C = Chezy coefficient of friction [m^(1/2)/s];
% Qin = Inflow hydrograph [m^3/s];
% Dx = Spatial step [m];
% Dt = Time step [s];
% nx = Number of spatial steps;
% nt = Number of time steps;
% dt = Time step of the inflow hydrograph [s];
% ntot = Number of time steps of the inflow hydrograph;
clc; clear all; close all;
% Input data;
g=9.81;
S=0.0004;
C=40;
Dx=1000;
Dt=5;
nx=46;
nt=50000;
Qin=2000;
ntot=50000;
% Import of filevw.mat: Channel widths;load filevw.mat
load filevw.mat
b=B(:);
% Initial conditions;
for i=1:nx
    h(i)=0.1;
    hmax(i)=0;
    u(i)=0;
end
k=0;
% Main program;
for k=1:nt
    k=k+1;
    % Calculation of linearly changing inflow hydrograph;
    Q(1)=Qin-Qin*k/ntot;
    if k-ntot>0
        Q(1)=0;
    end
    % Calculation of the velocity and discharge;
    for i=2:nx-1
        if h(i-1)-0.02<0
            u(i)=0;
        else u(i)=u(i)+Dt*g*(S-(h(i)-h(i-1))/Dx-u(i)*abs(u(i))/C^2*2/(h(i)+h(i-1)));
        end
        Q(i)=u(i)*b(i)*(h(i)+h(i-1))/2;
    end
    % Downstream boundary condition;
    Q(nx)=b(nx)*h(nx-1)*sqrt(g*h(nx-1));
    % Estimation of the water depths:
    for i=1:nx-1
        h(i)=h(i)-Dt/Dx/(b(i)+b(i+1))*2*(Q(i+1)-Q(i));
    end
    % Data stored at different time intervals for plotting;
    if k == round(nt/25)
        for m=1:nx
            h1t(m)=h(m);
            Q1t(m)=Q(m);
        end
    end
    if k == round(nt/5)
        for m=1:nx
            h3t(m)=h(m);
            Q3t(m)=Q(m);
        end
    end
    if k == round(nt)
        for m=1:nx
            h5t(m)=h(m);
            Q5t(m)=Q(m);
        end
    end
end
for j=1:ntot
    Qinflow(j)=Qin-Qin*j/ntot;
end
plot(1:ntot,Qinflow,'k','Linewidth',1.5)
xlabel('Time*Dt [s]')
ylabel('Inflow hydrograph [m^3/s]')
axis([0,51000,0,2100])
text(10000,200, 'Td=Flood duration: 5*10^4*Dt [s]')
figure, plot(1:nx,Q1t,'b','Linewidth',1.5)
hold on
plot(1:nx,Q3t,'r','Linewidth',1.5)
plot(1:nx,Q5t,'g','Linewidth',1.5)
xlabel('Longitudinal distance x 1000 [m]')
ylabel('Water discharge [m^3/s]')
axis([0,47,0,2000])
legend('t=Td/25','t=Td/5','t=Td')
figure, plot(1:nx,h1t,'b','Linewidth',1.5)
hold on
plot(1:nx,h3t,'r','Linewidth',1.5)
plot(1:nx,h5t,'g','Linewidth',1.5)
xlabel('Longitudinal distance x 1000 [m]')
ylabel('Water depth [m]')
axis([0,47,0,4.5])
legend('t=Td/25','t=Td/5','t=Td')

