% Example 5.4 Longshore Current Past a Marina Breakwater
% ho = Water depth [m];
% u = Velocity in the x-direction [m/s];
% v = Velocity in the y-direction [m/s];
% cv = Longshore current velocity [m/s];
% ev = Eddy viscosity [m^2/s];
% sp = Smagorinsky parameter for horizontal eddy coefficient [dimensionless];
% cf = Coriolis coefficient [1/s];
% fb = bed friction [dimensionless];
% Dd = Spatial step in both x and y directions [m];
% Dt = Time step [s];
% nx = Number of computational steps in the x-direction;
% ny = Number of computational steps in the y-direction;
% nt = Number of time steps;
clc; clear all; close all;
% Input data;
g=9.81;
ho=4;
cv=0.2;
sp=0.3;
cf=0.0001;
fb=0.001;
Dd=5;
Dt=0.5;
nx=70;
ny=40;
nt=14400;
load depths.txt
df=depths;
% Subroutine h=fm(i,j,df): For definition of the marina breakwater;
% Initial conditions;
for i=1:nx
    for j=1:ny
        u(i,j)=0;
        un(i,j)=0;
        v(i,j)=0;
        vn(i,j)=0;
        z(i,j)=0;
    end
end
% Main program;
for k=1:nt
    % Estimation of the eddy viscosity;
    for i=2:nx-1
        for j=2:ny-1
            t1=(u(i+1,j)-u(i,j))/Dd-(v(i,j+1)-v(i,j))/Dd;
            t2=(u(i,j+1)+u(i+1,j+1)-u(i,j-1)-u(i+1,j-1))/2/Dd+(v(i+1,j+1)+v(i+1,j)-v(i-1,j+1)-v(i-1,j))/4/Dd;
            ev(i,j)=sp*Dd^2*sqrt(t1^2+t2^2);
            if ev(i,j) < 0.05;
                ev(i,j)=0.05;
            end
        end
    end
    % Solution of the continuity equation;
    for i=2:nx-2
        for j=2:ny-2
            h1=fm(i,j,df);
            h2=fm(i-1,j,df);
            h3=fm(i+1,j,df);
            h4=fm(i,j+1,df);
            h5=fm(i,j-1,df);
            if h1>0.1
                hl=(h1+h2)/2;
                hr=(h1+h3)/2;
                ho=(h4+h1)/2;
                hu=(h1+h5)/2;
                z(i,j)=z(i,j)-Dt/Dd*(u(i+1,j)*hr-u(i,j)*hl+v(i,j+1)*ho-v(i,j)*hu);
            end
        end
    end
    % Solution x-axis equilibrium equation;
    for i=3:nx-2
        for j=2:ny-2
            evd=(ev(i,j)+ev(i-1,j))/2;
            h1=fm(i,j,df);
            h2=fm(i-1,j,df);
            if h1>0 && h2>0
                hm=(h1+h2)/2;
                vm=(v(i,j)+v(i,j+1)+v(i-1,j)+v(i-1,j+1))/4;
                adv=-u(i,j)*(u(i+1,j)-u(i-1,j))/2/Dd-vm*(u(i,j+1)-u(i,j-1))/2/Dd;
                cor=vm*cf;
                frict=-fb*sqrt(u(i,j)^2+vm^2)*u(i,j)/hm;
                pres=-g*(z(i,j)-z(i-1,j))/Dd;
                diff=evd*(-4*u(i,j)+u(i+1,j)+u(i-1,j)+u(i,j+1)+u(i,j-1))/Dd^2;
                un(i,j)=u(i,j)+Dt*(cor+pres+frict+diff+adv);
            end
        end
    end
    % Boundary conditions for velocity u;
    for j=2:ny-2
        h8=fm(2,j,df);
        if h8>0.1
            % Constant flow boundary condition;
            un(2,j)=cv;
        end
        h9=fm(nx-2,j,df);
        if h9>0.1
            % Open sea boundary condition;
            un(nx-1,j)=z(nx-2,j)*sqrt(g/h9);
        end
    end
    for i=2:nx-2
        un(i,1)=un(i,2);
        un(i,ny-1)=un(i,ny-2);
    end
    % Solution of the y-axis equilibrium condition;
    for i=2:nx-2
        for j=3:ny-1
            edd=(ev(i,j)+ev(i,j-1))/2;
            h1=fm(i,j,df);
            h5=fm(i,j-1,df);
            if h1>0 && h5>0
                hm=(h1+h5)/2;
                um=(u(i,j)+u(i+1,j)+u(i,j-1)+u(i+1,j-1))/4;
                adv=-v(i,j)*(v(i,j+1)-v(i,j-1))/2/Dd-um*(v(i+1,j)-v(i-1,j))/2/Dd;
                cor=-um*cf;
                frict=-fb*sqrt(v(i,j)^2+um^2)*v(i,j)/hm;
                pres=-g*(z(i,j)-z(i,j-1))/Dd;
                diff=evd*(-4*v(i,j)+v(i,j-1)+v(i,j+1)+v(i-1,j)+v(i+1,j))/Dd^2;
                vn(i,j)=v(i,j)+Dt*(cor+pres+frict+diff+adv);
            end
        end
    end
    % Open sea boundary conditions for velocity v:
    for i=2:nx-2
        h6=fm(i,2,df);
        h7=fm(i,ny-2,df);
        if h6<0.1
            if h7<0.1
                continue;
            else
                vn(i,ny-1)=z(i,ny-2)*sqrt(g/h7);
            end
        end
        vn(i,2)=-z(i,2)*sqrt(g/h6);
    end
    for j=2:ny-2
        vn(1,j)=vn(2,j);
        vn(nx-1,j)=vn(nx-2,j);
    end
    % Updating the velocities u and v;
    for i=1:nx
        for j=1:ny
            u(i,j)=un(i,j);
            v(i,j)=vn(i,j);
            % Zero flux boundary condition;
        end
    end
    % Calculation of the kinetic energy;
    ke=0;
    for i=1:nx
        for j=1:ny
            ke=ke+u(i,j)^2+v(i,j)^2;
        end
    end
    KinE(k)=ke;
%     index=k
end
plot(1:k,KinE,'Color','k','Linewidth',1.5)
xlabel('Number of time steps')
ylabel('Total kinetic energy (u^2+v^2 on all grid points)')
[i,j]=meshgrid(1:1:nx,1:1:ny);
df1=df(:,3);
df2=reshape(df1,70,40);
df2=df2';
An=ones(size(df2));
idx=find(df2==0);
An(idx)=0;
figure,pcolor(An); hold on;
colormap(bone)
shading flat
% Plot every interval;
% quiver(i,j,u',v','Color','k');
% Plot every second interval;
quiver(i(1:2:end,1:2:end),j(1:2:end,1:2:end),(u(1:2:end,1:2:end))',(v(1:2:end,1:2:end))','Color','k'); hold on;
