% Example 6.1 One Dimensional Linear Long Wave Over a Sloping Plane Bed
% ho = Water depth [m];
% ev = Eddy viscosity [m^2/s];
% ec = Coefficient for the eddy viscosity relation [dimensionless];
% fb = Bed friction [s^(-1)];
% a = Incident wave amplitude [m];
% T = Wave period [seconds];
% Dx = Spatial step [m];
% Dt = Time step [s];
% nx = Number of computational steps;
% nt = Number of time steps;
clc; clear all; close all;
% Input data;
g=9.81;
ho=5;
fbo=0.001;
a=1;
T=8;
ec=0.4;
Dx=2;
Dt=0.1;
nx=200;
nt=3040;
r=g*(Dt/Dx)^2/2;
nx2=nx/2;
% Option for variable bed friction;
for i=1:nx
    fb(i)=fbo;
end
% Bathymetry data for sloping bed;
for i=1:nx
    % For horizontal bed active next line;
    % h(i)=ho;
    % For sloping bed activate next line;
    h(i)=ho-(ho-.5)*i/nx;
    bed(i)=5-h(i);
end
for i=1:nx
    z(i)=0;
    zo(i)=0;
    zm(i)=0;
end
% Estimation of the eddy viscosity;
for i=1:nx
    ev(i)=0;
    if a>h(i)/2.5
        ev(i)=ec*h(i)*sqrt(g*h(i));
    end
end
c=sqrt(g*h(1));
L=c*T;
zr1=0;
% Main program;
for k=1:nt
    % Open sea boundary conditions: Incident and back radiated wave;
    zr2=z(2)-a*sin(2*pi*((k-1)*Dt/T-Dx/L));
    zr1=zr1+Dt/Dx*c*(zr2-zr1);
    zn(1)=a*sin(2*pi*k*Dt/T)+zr1;
    % Solution within the main domain;
    for i=2:nx-1
        br=ev(i)*Dt/Dx^2*(z(i+1)-2*z(i)+z(i-1)-zo(i+1)+2*zo(i)-zo(i-1));
        tt=r*((h(i+1)+h(i))*(z(i+1)-z(i))-(h(i)+h(i-1))*(z(i)-z(i-1)));
        zn(i)=2*z(i)-zo(i)+tt-Dt*fb(i)*(z(i)-zo(i))+br;
    end
    % Boundary condition at the coastline: Back radiated wave;
    zn(nx)=z(nx)+sqrt(g*h(nx))*Dt/Dx*(z(nx-1)-z(nx));
    for i=1:nx
        zo(i)=z(i);
        z(i)=zn(i);
        % Water depth below the water surface;
        hwater(i)=z(i)+ho;
    end
    if k>nx2
        % Estimation of the variance (h-rms)^2 (Wave height root-mean-square);
        for i=1:nx
            zm(i)=zm(i)+z(i)^2/nx;
        end
    end
end
plot(1:nx,hwater,'b','Linewidth',1.5)
hold on
plot(1:nx,zm,'g','LineWidth',1.5)
plot(1:nx,bed,'m','LineWidth',2)
plot(1:nx,ev,'r','LineWidth',1.5)
xlabel('Number of spatial steps x 2 [m]')
ylabel('[m], [m^2], [m^2/s]')
text(143,6,'Wave height [m]')
text(20,9.2,'(h-rms)^2 [m^2]')
text(20,2.7,'Time = nt*Dt [s]')
text(115,9.5,'Wave-breaking location')
text(40,1.5,'Bed')
text(112,0.5,'Eddy viscosity [m^2/s]')
