% Example 6.4 Two Dimensional Long Wave with Reflecting Breakwater
% a = Wave amplitude [m];
% T = Wave period [T];
% theta = Incident wave angle [degrees];
% rf = Reflection coefficient (rf = 1 for full reflection);
% ec = Parameter for the eddy viscosity;
% Dd = Spatial step in both x and y directions [m];
% ho = Water depth at the open sea boundary;
% Dt = Time step [s];
% nx = Number of computational steps in the x-direction;
% ny = Number of computational steps in the y-direction;
% nt = Number of computational time steps;
clc; clear all, close all;
% input data;
g=9.81;
a=0.5;
T=12;
theta=1;
rf=0.5;
ec=0.8;
ho=8.5;
Dd=3;
Dt=0.1;
nx=60;
ny=80;
nt=2400;
% Initialization of variable;
for i=1:nx
    for j=1:ny
        h(i,j)=0;
        z(i,j)=0;
        zo(i,j)=0;
        u(i,j)=0;
        v(i,j)=0;
        sxx(i,j)=0;
        syy(i,j)=0;
        sxy(i,j)=0;
        hrms(i,j)=0;
    end
end
ar=0;
al=0;
ao=0;
au=0;
chk=0;
% Water depth data;
for j=1:ny
    for i=2:nx-1
        h(i,j)=ho-j/ny*8;
    end
end
% Definition of the breakwater location and geometry;
for i=nx/4:3*nx/4
    h(i,ny/2.5)=0;
end
c1=sqrt(g*h(2,1));
b=theta*pi/180;
elo=c1*T/sin(b);
phi=2*pi*Dd/elo;
% Estimation of eddy coefficient in wave breaking zone;
for j=1:ny
    for i=1:nx
        if h(i,j)> 2.5*a
            ed(i,j)=0;
        else
            ed(i,j)=ec*h(i,j)*sqrt(g*h(i,j));
        end
    end
end
% Main program;
for k=1:nt
    % Open sea boundary condition: Incident and radiating waves;
    for i=1:nx
        zr1=z(i,1)-a*sin(2*pi*((k-1)*Dt/T)+(i-1)*phi);
        zr2=z(i,2)-a*sin(2*pi*((k-1)*Dt-Dd/c1)/T+(i-1)*phi);
        zr1=zr1+Dt/Dd*c1*(zr2-zr1);
        zn(i,1)=zr1+a*sin(2*pi*k*Dt/T+(i-1)*phi);
    end
    % Solution over the flow domain;
    for j=2:ny-1
        for i=2:nx-1
            if h(i,j)> 0.1
                al=z(i-1,j);
                chk=1000;
            end
            if chk==1000
                if h(i-1,j)< 0.1
                    al=z(i,j)*rf+(1-rf)*zo(i,j);
                end
                ar=z(i+1,j);
                if h(i+1,j)< 0.1
                    ar=z(i,j)*rf+(1-rf)*zo(i,j);
                end
                ao=z(i,j+1);
                if h(i,j+1)< 0.1
                    ao=z(i,j)*rf+(1-rf)*zo(i,j);
                end
                au=z(i,j-1);
                if h(i,j-1)< 0.1
                    au=z(i,j)*rf+(1-rf)*zo(i,j);
                end
                hr=(h(i+1,j)+h(i,j))/2;
                hl=(h(i-1,j)+h(i,j))/2;
                ho=(h(i,j+1)+h(i,j))/2;
                hu=(h(i,j)+h(i,j-1))/2;
                zn(i,j)=z(i,j)*2-zo(i,j)+(Dt/Dd)^2*g*(hr*(ar-z(i,j))-hl*(z(i,j)-al)+ho*(ao-z(i,j))-hu*(z(i,j)-au));
                zn(i,j)=zn(i,j)+ed(i,j)*Dt/Dd^2*(z(i+1,j)+z(i-1,j)+z(i,j+1)+z(i,j-1)-4*z(i,j)-zo(i+1,j)-zo(i-1,j)-zo(i,j+1)-zo(i,j-1)+4*zo(i,j));
                if abs(zn(i,j)) < 0.00001
                    zn(i,j)=0;
                end
            end
            chk=0;
        end
    end
    % Coastline boundary condition;
    for i=2:nx-1
        if h(i,ny)> 0.1
            zn(i,ny)=z(i,ny)+Dt/Dd*sqrt(g*h(i,ny))*(z(i,ny-1)-z(i,ny));
        end
    end
    % Left and right boundary conditions: Reflection or free radiation;
    for j=1:ny
        if h(1,j)>0.1
            % Reflection;
            % zn(1,j)=zn(2,j);
            % Free radiation;
            zn(1,j)=z(1,j)+Dt/Dd*sqrt(g*h(1,j))*(z(2,j)-z(1,j));
        end
        if h(nx,j)> 0.1
            % zn(nx,j)=zn(nx-1,j);
            zn(nx,j)=z(nx,j)+Dt/Dd*sqrt(g*h(nx,j))*(z(nx-1,j)-z(nx,j));
        end
    end
    % Updating the z wave values;
    for j=1:ny
        for i=1:nx
            zo(i,j)=z(i,j);
            z(i,j)=zn(i,j);
        end
    end
    % Free surface time series at the middle of the field;
    zmid(k)=z(nx/2,ny/2);
    if k>nt/2
        % Estimation of flow velocity field and radiation stresses;
        for j=2:ny-1
            for i=2:nx-1
                if h(i,j)>0.1 && h(i-1,j)>0.1 && h(i+1,j)>0.1
                    u(i,j)=u(i,j)-Dt/Dd*g*(z(i+1,j)-z(i-1,j))/2;
                end
            end
        end
        for j=2:ny-1
            for i=2:nx-1
                if h(i,j)>0.1 && h(i,j-1)>0.1 && h(i,j+1)>0.1
                    v(i,j)=v(i,j)-Dt/Dd*g*(z(i,j+1)-z(i,j-1))/2;
                end
            end
        end
        for j=2:ny-1
            for i=2:nx-1
                if h(i,j)>0.1
                    sxx(i,j)=sxx(i,j)+h(i,j)*u(i,j)^2+g/2*z(i,j)^2;
                    syy(i,j)=syy(i,j)+h(i,j)*v(i,j)^2+g/2*z(i,j)^2;
                    sxy(i,j)=sxy(i,j)+h(i,j)*u(i,j)*v(i,j);
                end
            end
        end
        % Computation of the rms wave height;
        for j=1:ny
            for i=1:nx
                hrms(i,j)=hrms(i,j)+z(i,j)^2;
            end
        end
    end
    index=k
end
for j=1:ny
    for i=1:nx
        hrms(i,j)=sqrt(hrms(i,j)/nx*2)*2;
        if h(i,j)<0.1
            hrms(i,j)=4;
        end
    end
end
plot(1:nt,zmid,'k','Linewidth',1.5)
xlabel('Time x 10^-^1 [s]')
ylabel('Free-surface oscillation : Midpoint of solution domain [m]')
figure
i=1:nx;
j=1:ny;
fplot=z(i,j);
fplot1=hrms(i,j);
surf(j,i,fplot)
figure
surf(j,i,fplot1)
figure
[i,j]=meshgrid(1:1:nx,1:1:ny);
pcolor(i,j,h'); hold on;
%colormap(gray(2))
shading flat
% Plotting at every node;
% quiver(i,j,u',v')
% Plotting at every four points;
quiver(i(1:4:end,1:4:end),j(1:4:end,1:4:end),(u(1:4:end,1:4:end))',(v(1:4:end,1:4:end))','Color','w'); hold on;

