% Example 6.5 Impulse Response of a Harbor Basin
% ho = Water depth [m];
% zo = Excitation amplitude [m];
% fb = Bed friction [dimensionless];
% fsero = Number of Fourier series terms;
% Dx = Computational spatial step [m];
% Dt = Computational time step [s];
% nx = Number of spatial steps;
% nt = Number of time steps;
clc; clear all; close all;
% Input data;
g=9.81;
ho=5;
zo=.5;
fb=0.001;
fsero=40;
Dx=10;
Dt=0.5;
nx=100;
nt=2000;
% Initialization of variables;
for i=1:nx
    z(i)=0;
    h(i)=0;
    u(i)=0;
    un(i)=0;
end
% Definition of constant depth;
for i=2:nx-2
    h(i)=ho;
end
z(nx-2)=zo;
% Main program;
for k=1:nt
    for i=2:nx-2
        z(i)=z(i)-Dt/Dx*((h(i)+h(i+1))*u(i+1)-(h(i)+h(i-1))*u(i))/2;
    end
    % Definition of the spatial location to be analyzed;
    % Note that nx/2 and nx/5 must be integers;
    x(k)=z(nx/2);
    x1(k)=z(nx/5);
    for i=3:nx-2
        if u(i)<0
            adv=u(i)*(u(i+1)-u(i))/Dx*Dt;
        else
            adv=u(i)*(u(i)-u(i-1))/Dx*Dt;
        end
        un(i)=u(i)-adv-Dt/Dx*g*(z(i)-z(i-1))-Dt*fb*u(i)*abs(u(i))/(h(i)+h(i+1))*2;
    end
    if h(nx-1)>0.1
        un(nx-1)=z(nx-2)*sqrt(g/h(nx-2));
    end
    for i=2:nx-1
        u(i)=un(i);
    end
end
% Fourier series analysis at two selected spatial locations;
fser=fsero;
av=0;
av1=0;
for k=1:nt
    av=av+x(k);
    av1=av1+x1(k);
end
av=av/nt;
av1=av1/nt;
for k=1:nt
    x(k)=x(k)-av;
    x1(k)=x1(k)-av1;
end
% Fourier coefficient a;
for m=1:fser
    a(m)=0;
    a1(m)=0;
    for k=1:nt
        a(m)=a(m)+x(k)*sin(2*pi*m*(k-1)/nt);
        a1(m)=a1(m)+x1(k)*sin(2*pi*m*(k-1)/nt);
    end
    a(m)=a(m)*2/nt;
    a1(m)=a1(m)*2/nt;
end
% Fourier coefficient b;
for m=1:fser
    b(m)=0;
    b1(m)=0;
    for k=1:nt
        b(m)=b(m)+x(k)*cos(2*pi*m*(k-1)/nt);
        b1(m)=b1(m)+x1(k)*cos(2*pi*m*(k-1)/nt);
    end
    b(m)=b(m)*2/nt;
    b1(m)=b1(m)*2/nt;
end
amax=0;
% Fourier amplitude periodogram;
for m=1:fser
    ampl(m)=sqrt(a(m)^2+b(m)^2);
    ampl1(m)=sqrt(a1(m)^2+b1(m)^2);
end
plot(1:nt,x,'b','Linewidth',1.0)
xlabel('Time steps x 0.5 [s]');
ylabel('Free surface excitation at mid-point of the basin [m]')
figure
plot(1:fser,a,'b','Linewidth',1.5);
hold on
plot(1:fser,b,'r','Linewidth',1.5);
xlabel('Wave number of Fourier series terms');
ylabel('Fourier series coefficients at mid-point of the basin');
legend('Coefficient a','Coefficient b',2);
figure
plot(1:fser,ampl,'b','Linewidth',1.5);
hold on
plot(1:fser,ampl1,'r','Linewidth',1.5);
xlabel('Wave number of Fourier series terms');
ylabel('Amplitude of Fourier components');
legend('Location x=nx/2','Location x=nx/5',2);
text(20,0.0080,'Periodogram')


