% Example 7.2 Unconfined Aquifer with Pumping Well
% h = Water table elevation [m];
% zb(j,j) = Elevation of the bed from the reference datum [m];
% h(i,j) = Water surface elevation [m];
% K = Permeability coefficient [m/hr];
% p = Porosity;
% n = Node parameter (interior, imperveous, constant head, well site);
% Dd = Spatial step (the same in both x and y directions [m];
% Dt = Time step [hr];
% nx = Number of steps in the x-direction;
% ny = Number of steps in the y-direction;
clc; clear all; close all;
% Input data;
Dd=500;
Dt=24;
nx=30;
ny=30;
nt=7200;
K=0.5;
p=0.2;
rr=Dt/Dd^2/2*K/p;
% Initialization of variables;
for i=1:nx
    for j=1:ny
        h(i,j)=50;
        zb(i,j)=0;
        % n=0 indicates the interior nodes to be calculated;
        n(i,j)=0;
    end
end
nxm=nx/2;
nym=ny/2;
% Definition of the boundary conditions;
for j=1:ny
    % n=1 indicates impervious boundaries;
    n(1,j)=1;
    n(nx,j)=1;
end
for i=1:nx
    n(i,1)=1;
    % n different than 0 and 1 indicates Piezometric head in meters;
    n(i,ny)=50;
end
% Definition of the well location and head;
% n at the well site indicates flow in m^3/hr;
n(nxm,nym)=-400;
for i=1:nx
    for j=1:ny
        hn(i,j)=h(i,j);
    end
end
chk=0;
% Main program;
for k=1:nt
    for j=1:ny
        for i=1:nx
            if n(i,j)>0
                % chk is a number used for computational purposes;
                chk=1000;
            end
            if chk ~= 1000
                fo=h(i,j+1);
                fu=h(i,j-1);
                fr=h(i+1,j);
                fl=h(i-1,j);
                % Impervious boundary conditions;
                if n(i,j+1)==1
                    fo=h(i,j);
                end
                if n(i,j-1)==1
                    fu=h(i,j);
                end
                if n(i+1,j)==1
                    fr=h(i,j);
                end
                if n(i-1,j)==1
                    fl=h(i,j);
                end
                % Constant head boundary conditions;
                if n(i,j+1)>1
                    fo=n(i,j+1);
                end
                if n(i,j-1)>1
                    fu=n(i,j-1);
                end
                if n(i+1,j)>1
                    fr=n(i+1,j);
                end
                if n(i-1,j)>1
                    fl=n(i-1,j);
                end
                dr=fr-zb(i+1,j);
                dl=fl-zb(i-1,j);
                do=fo-zb(i,j+1);
                du=fu-zb(i,j-1);
                dc=h(i,j)-zb(i,j);
                fc=h(i,j);
                Qwell=0;
                if n(i,j)<0
                    Qwell=n(i,j)/Dd^2/p;
                end
                hn(i,j)=h(i,j)+rr*((dr+dc)*(fr-fc)-(dc+dl)*(fc-fl)+(do+dc)*(fo-fc)-(dc+du)*(fc-fu));
                hn(i,j)=hn(i,j)+Dt*Qwell;
            end
            chk=0;
        end
    end
    for i=1:nx
        for j=1:ny
            h(i,j)=hn(i,j);
        end
    end
    test=k
    WellDepth(k)=h(nxm,nym);
end
m=1:nt;
WD=WellDepth(m);
figure, plot(m,WD,'b','Linewidth',1.5);
xlabel('Number of time steps');
ylabel('Water depth in the well [m]');
i=1:nx;
j=1:ny;
fpp=hn(i,j);
figure, surf(i,j,fpp)
figure
contourf(i,j,fpp)
