% Example 7.3 Salt Water Intrusion Due to Pumping Well
% Ho = Initial depth of the upper layer [m];
% Hu = Initial depth of the lower layer [m];
% qo = Point discharge of the upper layer [m^3/m/day];
% qu = Point discharge of the lower layer [m^3/m/day];
% p = Porosity;
% K = Permeability coefficient [m/day];
% rdf = Relative density difference;
% iwo = Location of well in upper layer;
% iwu = Location of well in lower layer;
% Dx = Spatial step [m];
% Dt = Time step [day];
% nx = Number of spatial steps;
% nt = Number of time steps;
clc; clear all; close all;
% Input data;
Ho=20.8;
Hu=20;
qo=-3;
qu=0;
p=0.4;
K=500;
rdf=0.02;
iwo=45;
iwu=0;
Dx=200;
Dt=0.1;
nx=50;
nt=800000;
% Initial depths in the two layers;
for i=2:nx-1
    ho(i)=Ho;
    hon(i)=ho(i);
    hu(i)=0;
    hun(i)=hu(i);
end
ho(nx)=Ho;
hon(nx)=Ho;
hu(1)=Hu;
hun(1)=Hu;
ho(1)=0;
hon(1)=ho(1);
hu(nx)=0;
hun(nx)=hu(nx);
for k=1:nt
    % Calculation of new depth values of the upper layer;
    for i=2:nx-1
        Qwo=0;
        if i==iwo
            Qwo=qo;
        end
        qr=-K*(ho(i+1)+hu(i+1)-ho(i)-hu(i))/Dx*(ho(i)+ho(i+1))/2;
        ql=-K*(ho(i)+hu(i)-ho(i-1)-hu(i-1))/Dx*(ho(i)+ho(i-1))/2;
        hon(i)=ho(i)-Dt*(qr-ql-Qwo)/Dx/p;
    end
    % Calculation of new depth values of the lower layer;
    for i=2:nx-1
        Qwu=0;
        if i==iwu
            Qwu=qu;
        end
        qr=-K*(ho(i+1)+hu(i+1)-ho(i)-hu(i)-rdf*(ho(i+1)-ho(i)))/Dx*(hu(i)+hu(i+1))/2;
        ql=-K*(ho(i)+hu(i)-ho(i-1)-hu(i-1)-rdf*(ho(i)-ho(i-1)))/Dx*(hu(i)+hu(i-1))/2;
        hun(i)=hu(i)-Dt*(qr-ql-Qwu)/Dx/p;
        if hun(i)<0
            hun(i)=0;
        end
    end
    % Boundary conditions;
    hun(nx)=hun(nx-1);
    % Updating depth values;
    for i=1:nx
        ho(i)=hon(i);
        hu(i)=hun(i);
    end
    chk=0;
    xtoe=0;
    % Estimation of saline wedge toe location;
    for i=1:nx
        if hu(i)<0.1
            xtoe=i;
            break
        end
    end
    toe(k)=xtoe;
    index=k
end
i=1:nx;
GWt=ho(i);
SWe=hu(i);
ground=GWt+SWe;
plot(1:k,toe,'b','Linewidth',1.5)
xlabel('Time steps x 0.1 [days]')
ylabel('Position of the saline wedge toe x 200 [meters]')
figure
plotyy(1:nx,SWe,1:nx,ground)
xlabel('Distance from the coast x 200 [meters]')
ylabel('Interface [meters]')
text(3,15,'Saline wedge');
text(25,8.8,'Fresh water table');
