% Example 8.2 Contaminant Advective Diffusion in Thermaikos Gulf
% csource = Pollution concentration at the source [kg/m^3];
% dc = Decay coefficient [s^-1];
% imm = x-axis coordinate of the pollution source;
% jmm = y-axis coordinate of the pollution source;
% Dd = Spatial step (same in both directions)[m];
% Dt = Time step {s};
% nx = Number of computational steps in the x-direction;
% ny = Number of computational steps in the y-direction;
% nt = Number of time steps;
clc; clear all; close all;
% Input data;
csource=1;
dc=0.00001;
imm=12;
jmm=12;
Dd=2000;
Dt=600;
nx=22;
ny=23;
nt=288;
% Initialization of variables;
for i=1:nx
    for j=1:ny
        c(i,j)=0;
        cmax(i,j)=0;
        cn(i,j)=0;
    end
end
% Importing data for depths, velocities and diffusion coefficients;
load ThermD.txt
load ThermU.txt
load ThermV.txt
load ThermE.txt
fDep=ThermD;
fCoU=ThermU;
fCoV=ThermV;
fEdd=ThermE;
% Subrouting for depths h=fm(i,j,fDep);
% Subroutine for velocity u=fm(i,j,fCoU);
% Subroutine for velocity v=fm(i,j,fCoV);
% Subroutine for diffusion coefficients ed=fm(i,j,fEdd);
% Main program;
for k=1:nt
    for j=2:ny-2
        for i=2:nx-2
            h=fm(i,j,fDep);
            ho=fm(i,j+1,fDep);
            hu=fm(i,j-1,fDep);
            hl=fm(i-1,j,fDep);
            hr=fm(i+1,j,fDep);
            u=fm(i,j,fCoU);
            ur=fm(i+1,j,fCoU);
            v=fm(i,j,fCoV);
            vo=fm(i,j+1,fCoV);
            ed=fm(i,j,fEdd);
            if h>0.1
                advx=0;
                if u+ur>0
                    advx=u*(c(i,j)-c(i-1,j))/Dd;
                else
                    advx=ur*(c(i+1,j)-c(i,j))/Dd;
                end
                advy=0;
                if v+vo>0
                    advy=v*(c(i,j)-c(i,j-1))/Dd;
                else
                    advy=vo*(c(i,j+1)-c(i,j))/Dd;
                end
                co=c(i,j+1);
                if ho<0.1
                    co=c(i,j);
                end
                cu=c(i,j-1);
                if hu<0.1
                    cu=c(i,j);
                end
                cr=c(i+1,j);
                if hr<0.1
                    cr=c(i,j);
                end
                cl=c(i-1,j);
                if hl<0.1
                    cl=c(i,j);
                end
                diff1=ed/h*((co-c(i,j))*(h+ho)-(c(i,j)-cu)*(h+hu))/2/Dd^2;
                diff2=ed/h*((cr-c(i,j))*(hr+h)-(c(i,j)-cl)*(h+hl))/2/Dd^2;
                cn(i,j)=c(i,j)+Dt*(-advx-advy+diff1+diff2-dc*c(i,j));
            end
        end
    end
    % Boundary conditions;
    for i=2:nx-1
        cn(i,2)=2*cn(i,3)-c(i,4);
    end
    %   for j=2:ny-1
    %       cn(2,j)=2*cn(3,j)-cn(4,j);
    %       cn(nx-2,j)=2*cn(nx-3,j)-cn(nx-4,j);
    %    end
    % Point source pollution;
    cn(imm,jmm)=csource;
    % Updating the contaminant concentration values;
    for j=1:ny
        for i=1:nx
            c(i,j)=cn(i,j);
            if cn(i,j)>cmax(i,j)
                cmax(i,j)=cn(i,j);
            end
        end
    end
    index=k
end
[i,j]=meshgrid(1:1:nx,1:1:ny);
df1=fDep(:,3);
df2=reshape(df1,22,23);
df2=df2';
An=ones(size(df2));
idx=find(df2==0);
An(idx)=0;
figure;
pcolor(An); hold on;
colormap(jet)
shading flat
i=1:nx;
j=1:ny;
Cplot=cmax(i,j);
contour(i,j,Cplot')
% colorbar('vert')
















