% 8.3 Advective Diffusion by Random Particle Motion - Marina Breakwater
% dif = Diffusion coefficient;
% dec = Decay coefficient;
% ipp = Number of simulated particles;
% xo, yo = Initial position of particles [m];
% Dd = Spatial computational step [m];
% Dt = Time computational step [s];
% nx = Number of computational steps in the x-direction;
% ny = Number of computational steps in the y-direction;
% nt = Number of computational steps in time;
clc; clear all; close all;
% Input data
dif=0.2;
dec=0.00001;
ipp=5000;
xo=90;
yo=125;
Dd=5;
Dt=5;
nx=70;
ny=40;
nt=720;
% Importing data for depths, velocities and diffusion coefficients;
load Depths.txt
load CoastU.txt
load CoastV.txt
fDep=Depths;
fCoU=CoastU;
fCoV=CoastV;
% Subroutine for depths h=fm(i,j,fDep);
% Subroutine for velocity u=fm(i,j,fCoU);
% Subroutine for velocity v=fm(i,j,fCoV);
for i=1:nx
    for   j=1:ny
        cmax(i,j)=0;
    end
end
imm=(nx-1)*Dd;
jmm=(ny-1)*Dd;
% Initial position of particles;
for i=1:ipp
    x(i)=xo;
    y(i)=yo;
end
% Number of decaying particles on every time step;
depp=round(ipp*Dt*dec);
diff=sqrt(6*dif/Dt);
% Main program;
for k=1:nt
    % Movement of particles by advection and diffusion;
    for i=1:ipp
        X=fix(x(i)/Dd)+1;
        Y=fix(y(i)/Dd)+1;
        if X>2 && X<nx-1 && Y>2 && Y<ny-1
            u=fm(X,Y,fCoU);
            ur=fm(X+1,Y,fCoU);
            v=fm(X,Y,fCoV);
            vo=fm(X,Y+1,fCoV);
            uu=(u+ur)/2;
            vv=(v+vo)/2;
            tempx=x(i);
            tempy=y(i);
            ax=rand;
            x(i)=x(i)+(2*(ax-0.5)*diff+uu)*Dt;
            ay=rand;
            y(i)=y(i)+(2*(ay-0.5)*diff+vv)*Dt;
            X=fix(x(i)/Dd)+1;
            Y=fix(y(i)/Dd)+1;
            % Deflection on solid boundarues;
            h=fm(X,Y,fDep);
            if h<0.1
                x(i)=tempx;
                y(i)=tempy;
            end
        end
    end
    % Decaying of particles;
    if dec>0
        for i=1:depp
            ad=rand;
            X=fix(ad*ipp)+1;
            x(X)=0;
            y(Y)=0;
        end
    end
    % Computation of particle concentration in grid cells;
    for i=1:nx
        for j=1:ny
            c(i,j)=0;
        end
    end
    for i=1:ipp
        X=fix(x(i)/Dd)+1;
        Y=fix(y(i)/Dd)+1;
        c(X,Y)=c(X,Y)+1;
    end
    for j=1:ny
        for i=1:nx
            if c(i,j)>cmax(i,j)
                cmax(i,j)=c(i,j);
            end
        end
    end
    index=k
end
[i,j]=meshgrid(1:1:nx,1:1:ny);
df1=fDep(:,3);
df2=reshape(df1,70,40);
df2=df2';
An=ones(size(df2));
idx=find(df2==0);
An(idx)=0;
figure;
pcolor(An);
colormap(jet);
shading flat;
hold on;
i=1:nx;
j=1:ny;
Cplot=cmax(i,j)/ipp;
contour(i,j,Cplot')
% colorbar('vert')


