% 8.4 Advective Diffusion by Random Particle Motion - Thermaikos Gulf
% dif = Diffusion coefficient;
% dec = Decay coefficient;
% ipp = Number of simulated particles;
% xo, yo = Initial position of particles [m];
% Dd = Spatial computational step [m];
% Dt = Time computational step [s];
% nx = Number of computational steps in the x-direction;
% ny = Number of computational steps in the y-direction;
% nt = Number of computational steps in time;
clc; clear all; close all;
% Input data
dif=5;
dec=0.00001;
ipp=5000;
xo=24000;
yo=24000;
Dd=2000;
Dt=600;
nx=22;
ny=23;
nt=288;
% Importing data for depths, velocities and diffusion coefficients;
load ThermD.txt
load ThermU.txt
load ThermV.txt
fDep=ThermD;
fCoU=ThermU;
fCoV=ThermV;
% Subroutine for depths h=fm(i,j,fDep);
% Subroutine for velocity u=fm(i,j,fCoU);
% Subroutine for velocity v=fm(i,j,fCoV);
for i=1:nx
    for   j=1:ny
        cmax(i,j)=0;
    end
end
imm=(nx-1)*Dd;
jmm=(ny-1)*Dd;
% Initial position of particles;
for i=1:ipp
    x(i,1)=xo;
    y(i,1)=yo;
end
% Number of decaying particles on every time step;
depp=round(ipp*Dt*dec);
diff=sqrt(6*dif/Dt);
% Main program;
for k=1:nt
    % Movement of particles by advection and diffusion;
    for i=1:ipp
        X=fix(x(i,1)/Dd)+1;
        Y=fix(y(i,1)/Dd)+1;
        if X>2 && X<nx-1 && Y>2 && Y<ny-1
            u=fm(X,Y,fCoU);
            ur=fm(X+1,Y,fCoU);
            v=fm(X,Y,fCoV);
            vo=fm(X,Y+1,fCoV);
            uu=(u+ur)/2;
            vv=(v+vo)/2;
            tempx=x(i,1);
            tempy=y(i,1);
            ax=rand;
            x(i,1)=x(i,1)+(2*(ax-0.5)*diff+uu)*Dt;
            ay=rand;
            y(i,1)=y(i,1)+(2*(ay-0.5)*diff+vv)*Dt;
            X=fix(x(i,1)/Dd)+1;
            Y=fix(y(i,1)/Dd)+1;
            % Deflection on solid boundarues;
            h=fm(X,Y,fDep);
            if h<0.1
                x(i,1)=tempx;
                y(i,1)=tempy;
            end
        end
    end
    % Decaying of particles;
    if dec>0
        for i=1:depp
            ad=rand;
            X=fix(ad*ipp)+1;
            x(X,1)=10;
            y(X,1)=50;
        end
    end
    % Computation of particle concentration in grid cells;
    for i=1:nx
        for j=1:ny
            c(i,j)=0;
        end
    end
    for i=1:ipp
        X=fix(x(i,1)/Dd)+1;
        Y=fix(y(i,1)/Dd)+1;
        c(X,Y)=c(X,Y)+1;
    end
    for j=1:ny
        for i=1:nx
            if c(i,j)>cmax(i,j)
                cmax(i,j)=c(i,j);
            end
        end
    end
    index=k
end
[i,j]=meshgrid(1:1:nx,1:1:ny);
df1=fDep(:,3);
df2=reshape(df1,22,23);
df2=df2';
An=ones(size(df2));
idx=find(df2==0);
An(idx)=0;
figure;
pcolor(An);
colormap(jet);
shading flat;
hold on;
i=1:nx;
j=1:ny;
Cplot=cmax(i,j)/ipp;
contour(i,j,Cplot')
% colorbar('vert')


