% Example 8.5 Nonlinear Sediment Transport Along a Dredged Canal
% ho = Initial water depth [m];
% qw = Water discharge per unit width [m^3/m/s]
% a = Sediment transport coefficient [s^2/m];
% Dx = Spatial step [m];
% Dt = Time step [s];
% nx = Number of spatial steps;
% nt = Number of temporal steps;
% Input data;
a=0.01;
ho=2;
qw=2.0;
Dx=2;
Dt=20;
nx=200;
nt=8640;
% Canal topography - Location and depth of dredge;
load Dredge.txt
for i=1:nx
    h(i)=Dredge(i,2);
end
% Initialization of variables;
for i=1:nx
    hn(i)=h(i);
end
% Main program;
for k=1:nt
    % Sediment transport and bed reshaping;
    for i=2:nx
        q(i)=a*qw^3/(h(i)+h(i-1))^3;
    end
    q(1)=q(2);
    for i=2:nx-2
        if q(i)+q(i+1)>0
            adv=(q(i)-q(i-1))/Dx;
        else
            adv=(q(i+2)-q(i+1))/Dx;
        end
        hn(i)=h(i)+Dt*adv;
    end
    hn(1)=hn(2);
    hn(nx-1)=hn(nx-2);
    for i=2:nx-2
        h(i)=hn(i);
    end
    h(1)=h(2);
    h(nx-1)=h(nx-2);
    % Storage of depths at time = 0;
    if k==1
        for i=1:nx
            ho(i)=h(i);
        end
    end
    % Storage of depths at half of the total simulation time;
    if k==nt/2
        for i=1:nx
            h1(i)=h(i);
        end
    end
end
i=1:nx;
hoplot=-ho(i);
h1plot=-h1(i);
hplot=-h(i);
plot(i,hoplot,'','Linewidth',1.5)
hold on
plot(i,h1plot,'r','Linewidth',1.5)
plot(i,hplot,'g','Linewidth',1.5)
xlabel('Longitudinal distance 2 x [m]')
ylabel('Depth of dredge below the bed [m]')
text(58,-3.8,'Time: t = 0')
text(75,-3.0,'Time: t = 1 day')
text(120,-2.6,'Time: t = 2 days')
