% Example 8.8 Cross Shore Sediment Transport and Beach Profile Formation
% H = Maximum water height at the left-hand boundary [m];
% ao = Wave height at the left-hand boundary [m];
% T = Wave period [s];
% S = Bed slope;
% dsed = Sediment density [kg/m^3];
% dwtr = Water density [kg/m^3];
% d50 = Mean particle diameter [m];
% ks = Absolute bed roughness [m];
% h() = Water depth [m];
% Dx = Spatial step [m];
% Dt = Time step [s];
% nt = Number of time steps;
clc; clear all; close all;
% Input data;
g=9.81;
H=5;
ao=2;
T=6;
S=0.02;
ks=0.001;
dsed=2.5;
dwtr=1.0;
d50=0.001;
Dx=3;
Dt=12;
nt=1500;
lo=g*T^2/(2*pi);
sed=g^2.5*((dsed/dwtr)-1)^2*d50;
% Definition of initial depths and solution domain size;
kx=1000;
for k=2:kx
    h(k)=H-S*(k-1)*Dx;
    if h(k)>0
        nxx=k;
    else
        nx=nxx-1;
        break
    end
end
% Breaking water depth;
hb=ao/0.8;
% Estimation of water setup and modified depths;
xsi=S/sqrt(ao/lo);
G=xsi^0.17+0.08;
set=hb/(1+8/3/G^2);
% Estimation of the breaking line location;
h(1)=H;
for i=2:nx
    h(i)=H-S*(i-1)*Dx;
    if h(i)<hb && h(i-1)>hb
        nb=i;
        break
    end
end
% New depths inside the breaker zone increased by setup;
for i=nb:nx
    h(i)=h(i)+set*(i-nb)/(nx-nb);
end
h(nx+1)=2*h(nx)-h(nx-1);
% Main program;
for k=1:nt
    hb=h(nb);
    % New wave breaking height at nb after the increase of water depth hb;
    ab=hb*G;
    % New wave distribution inside breaker zone;
    for i=1:nx
        if i<nb
            a(i)=ao;
        else
            a(i)=ab-(ab-set*G)*(i-nb)/(nx-nb);
        end
    end
    a(nx+1)=2*a(nx)-a(nx-1);
    % Estimation of wave characteristics along domain;
    temp=0;
    for i=1:nx
        l(i)=lo;
        while(abs(l(i)-temp)>0.1)
            temp=l(i);
            arg=2*pi*h(i)/l(i);
            l(i)=lo*(exp(arg)-exp(-arg))/(exp(arg)+exp(-arg));
            test=abs(l(i)-temp);
        end
    end
    l(nx+1)=2*l(nx)-l(nx);
    % Orbital and residual velocities; Shear stresses; Sediment discharges;
    for i=1:nx
        hmm=(h(i)+h(i+1))/2;
        amm=(a(i)+a(i+1))/2;
        lmm=(l(i)+l(i+1))/2;
        if i>nb
            arg=2*pi*hmm/lmm;
            uc(i)=-2.5*amm^2/T/(0.8*hmm);
            uo(i)=pi*amm/T/(exp(arg)-exp(-arg))*2;
        else
            arg=2*pi*hmm/lmm;
            cc=lmm/T;
            uc(i)=(pi*amm/T/(exp(arg)-exp(-arg))*2)^2/2/cc;
            uo(i)=pi*amm/T/(exp(arg)-exp(-arg))*2;
        end
        ch=18*log(12*hmm/ks);
        twc=g*uc(i)^2/ch^2+0.25*0.05*uo(i)^2;
        qs(i)=0.05*uc(i)*ch*twc^2/sed;
    end
    % Continuity equation for estimation of new depths;
    for i=2:nx
        hhb(i)=(qs(i)-qs(i-1))/Dx*Dt;
    end
    % Filtering the water depths by moving averages;
    for i=1:nx
        h(i)=h(i)+hhb(i);
    end
    for i=2:nx-1
        hm(i)=0.8*h(i)+0.1*(h(i-1)+h(i+1));
    end
    for i=2:nx-1
        h(i)=hm(i);
    end
    h(nx+1)=2*h(nx)-h(nx-1);
    %   index=k
    hbb(k)=h(nb);
    index=k
end
% Original beach profile;
x=0;
for k=1:83
    x=x+k;
    yplot(k)=-H+0.06*k;
end
ii=1:k;
i=1:nx;
hplot=-h(i);
plot(1:nx,hplot,'Linewidth',1.5)
xlabel('Longitudinal distance x 3 [m]')
ylabel('Beach profile [m]')
hold on;
plot(ii,yplot,'m','Linewidth',1.5)
text(30,-0.7,'Evolved beach profile')
text(50,-2.3,'Original beach profile')
figure
k=1:nt;
hbplot=hbb(k);
plot(1:nt,hbplot,'Linewidth',1.5)
xlabel('Time x 12 [s]')
ylabel('Water depth over the sandbar [m]')