% Example 8.9 Air Bubbles Induced Flow
% qg = Air discharge [m^3/s];
% wb = Bubbles/particle rising velocity [m/s];
% cp = Pseudo compressibility coefficient [m/s];
% xo, yo = Particles origin coordinates [m];
% tau = Drag force divided by the density per particle [m^3/s^2];
% ed = Eddy diffusion coefficient [m^2/s];
% ip = Total number of particles;
% Dd = Spatial step same in both directions [m];
% Dt = Time step [s];
% nx = Number of steps in the x-direction;
% ny = Number of steps in the y-direction;
% nt = Number of time steps;
clc; clear all; close all;
% Input data;
qg=0.001;
wb=0.35;
cp=4;
xo=14.5;
yo=1.5;
tau=0.000755;
ed=0.1;
ip=8000;
Dd=1;
Dt=0.05;
nx=30;
ny=30;
nt=4000;
% Initialization of variables;
for j=1:ny
    for i=1:nx
        h(i,j)=1;
        hn(i,j)=1;
        u(i,j)=0;
        un(i,j)=0;
        v(i,j)=0;
        vn(i,j)=0;
        c(i,j)=0;
        co(i,j)=0;
    end
end
uu=0;
vv=0;
% Number of particles entering the field per unit time;
ipp=fix(ip/nt);
vol=qg*Dt/ipp;
for i=1:ipp
    x(i)=xo;
    y(i)=yo;
end
for k=1:nt
    ipt=ipp*k;
    if ipt>ip
        ipt=ip;
    end
    D=sqrt(6*ed/Dt);
    for i=ipt-ipp+1:ipt
        x(i)=xo;
        y(i)=yo;
    end
    % Estimation of pressure head;
    for j=2:ny-2
        for i=2:nx-2
            hn(i,j)=h(i,j)-Dt/Dd/2*(u(i+1,j)-u(i,j)+v(i,j+1)-v(i,j))+(c(i,j)-co(i,j))*vol/Dd^2;
        end
    end
    for j=1:ny-1
        hn(1,j)=hn(2,j);
        hn(nx-1,j)=hn(nx-2,j);
    end
    for i=1:nx-1
        hn(i,1)=hn(i,2);
        hn(i,ny-1)=hn(i,ny-2);
    end
    for j=1:ny-1
        for i=1:nx-1
            h(i,j)=hn(i,j);
            co(i,j)=c(i,j);
        end
    end
    % Computation of velocities along the x-axis;
    for j=2:ny-2
        for i=3:nx-2
            un(i,j)=u(i,j)-Dt/Dd*cp^2*(h(i,j)-h(i-1,j))+ed*5*Dt/Dd^2*(u(i+1,j)+u(i-1,j)+u(i,j+1)+u(i,j-1)-4*u(i,j));
        end
    end
    for j=1:ny
        un(2,j)=-(h(2,j)-1)*cp/h(2,j);
        un(nx-1,j)=(h(nx-2,j)-1)*cp/h(nx-2,j);
    end
    for i=1:nx
        un(i,1)=un(i,2);
        un(i,ny-1)=un(i,ny-2);
    end
    % Computation of velocities along the y-axis;
    for j=3:ny-2
        for i=2:nx-2
            vn(i,j)=v(i,j)-Dt/Dd*cp^2*(h(i,j)-h(i,j-1))+ed*5*Dt/Dd^2*(v(i,j+1)+v(i,j-1)+v(i-1,j)+v(i+1,j)-4*v(i,j))+Dt*(c(i,j)+c(i,j-1))/2/Dd^2*tau;
        end
    end
    for j=1:ny
        vn(1,j)=vn(2,j);
        vn(nx-1,j)=vn(nx-2,j);
    end
    % Updating of the velocity values;
    for i=1:nx
        for j=1:ny
            u(i,j)=un(i,j);
            v(i,j)=vn(i,j);
        end
    end
    % Particle relocation by advection, buoyancy and diffusion;
    for i=1:ipt
        X=fix(x(i)/Dd)+1;
        Y=fix(y(i)/Dd)+1;
        if X<nx-1 && X>1 && Y<ny-1 && Y>1
            uu=u(X,Y)+(u(X+1,Y)-u(X,Y))*(x(i)-(X-1)*Dd)/Dd;
            vv=v(X,Y)+(v(X,Y+1)-v(X,Y))*(y(i)-(Y-1)*Dd)/Dd;
        end
        tempx=x(i);
        tempy=y(i);
        ax=rand;
        x(i)=x(i)+Dt*(uu+(2*ax-1)*D);
        ay=rand;
        y(i)=y(i)+Dt*(vv+(2*ay-1)*D+wb);
    end
    % Estimation of particle concentration in each cell;
    for j=1:ny-1
        for i=1:nx-1
            c(i,j)=0;
        end
    end
    for i=1:ipt
        X=fix(x(i)/Dd)+1;
        Y=fix(y(i)/Dd)+1;
        if X<nx-1 && X>1 && Y<ny-1 && Y>1
            c(X,Y)=c(X,Y)+1;
        end
    end
    % Estimation of the kinetic energy;
    KE=0;
    for i=2:nx-2
        for j=2:ny-2
            KE=KE+u(i,j)^2+v(i,j)^2;
        end
    end
    KinE(k)=KE;
    index=k
end
plot(1:k,KinE,'Linewidth',1.5);
xlabel('Time x 0.05 [s]')
ylabel('Kinetic energy u^2+v^2')
figure
[i,j]=meshgrid(1:1:nx,1:1:ny);
quiver(i,j,u',v');
xlabel('Horizontal distance [m]')
ylabel('Vertical distance [m]')
figure
contour(i,j,c')
xlabel('Horizontal distance [m]')
ylabel('Vertical distance [m]')